import { MouseCursor } from "../../interfaces/frontend"
// import Bluetooth from "./Bluetooth"
// import FogImages from "./FogImages"
import Labeling from "./Labeling"
import Ships from "./Ships"
import BorderTiles from "@components/Gamefield/BorderTiles"
import EventBar from "@components/Gamefield/EventBar"
import HitElems from "@components/Gamefield/HitElems"
import Targets from "@components/Gamefield/Targets"
import { useDraw } from "@hooks/useDraw"
import { useDrawProps } from "@hooks/useDrawProps"
import { useGameProps } from "@hooks/useGameProps"
import {
  initlialMouseCursor,
  overlapsWithAnyBorder,
  isAlreadyHit,
  targetList,
} from "@lib/utils/helpers"
import { CSSProperties, useCallback } from "react"
import { useEffect, useState } from "react"

export const count = 12

function Gamefield() {
  const [mouseCursor, setMouseCursor] =
    useState<MouseCursor>(initlialMouseCursor)
  const {
    hits,
    target,
    targetPreview,
    DispatchHits,
    setTarget,
    setTargetPreview,
  } = useGameProps()
  const [mode, setMode] = useState(0)

  const settingTarget = useCallback(
    (isGameTile: boolean, x: number, y: number) => {
      const list = targetList(targetPreview, mode)
      if (
        !isGameTile ||
        !list.filter(({ x, y }) => !isAlreadyHit(x, y, hits)).length
      )
        return
      if (target.show && target.x == x && target.y == y) {
        DispatchHits({
          type: "fireMissile",
          payload: list.map(({ x, y }) => ({
            hit: isAlreadyHit(x, y, hits),
            x,
            y,
          })),
        })
        setTarget((t) => ({ ...t, show: false }))
      } else if (!overlapsWithAnyBorder(targetPreview, mode))
        setTarget({ show: true, x, y })
    },
    [DispatchHits, hits, mode, setTarget, target, targetPreview]
  )

  useEffect(() => {
    const { x, y, show } = target
    const { shouldShow, ...position } = mouseCursor
    if (!shouldShow || overlapsWithAnyBorder(position, mode))
      setTargetPreview((e) => ({ ...e, show: false }))
    else {
      setTargetPreview({
        ...position,
        show: !show || x !== position.x || y !== position.y,
      })
    }
  }, [mode, mouseCursor, setTargetPreview, target])

  const { canvasRef, onMouseDown, clear } = useDraw()
  const { enable, color, shouldHide } = useDrawProps()

  return (
    <div id="gamefield">
      {/* <Bluetooth /> */}
      <div
        id="game-frame"
        style={{ "--i": count } as CSSProperties}
        onMouseLeave={() =>
          setMouseCursor((e) => ({ ...e, shouldShow: false }))
        }
      >
        {/* Bordes */}
        <BorderTiles props={{ settingTarget, setMouseCursor }} />

        {/* Collumn lettes and row numbers */}
        <Labeling count={count} />

        {/* Ships */}
        <Ships />
        <HitElems hits={hits} />

        {/* Fog images */}
        {/* <FogImages /> */}

        <Targets props={{ target, targetPreview, mode, hits }} />
        {/* <span id='dev-debug' style={{gridArea: '1 / 12 / 1 / 15', backgroundColor: 'red', zIndex: 3} as CSSProperties}>Debug</span> */}

        <canvas
          style={
            {
              opacity: shouldHide ? 0 : 1,
              boxShadow: enable ? "inset 0 0 0 2px " + color : "none",
              pointerEvents: enable && !shouldHide ? "auto" : "none",
            } as CSSProperties
          }
          ref={canvasRef}
          onMouseDown={onMouseDown}
          width="648"
          height="648"
        />
      </div>
      <EventBar props={{ setMode, clear }} />
    </div>
  )
}

export default Gamefield
